package com.qmth.themis.business.service.impl;

import com.qmth.themis.business.cache.ExamRecordCacheUtil;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.MqDto;
import com.qmth.themis.business.enums.ExamRecordStatusEnum;
import com.qmth.themis.business.enums.MqTagEnum;
import com.qmth.themis.business.service.CommonService;
import com.qmth.themis.business.service.MqDtoService;
import com.qmth.themis.business.service.TOeExamRecordService;
import com.qmth.themis.business.util.MqUtil;
import com.qmth.themis.business.util.RedisUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.Map;

/**
 * @Description:
 * @Author: xiatian
 * @Date: 2020-08-04
 */
@Service
public class CommonServiceImpl implements CommonService {
    private final static Logger log = LoggerFactory.getLogger(CommonService.class);

    @Resource
    TOeExamRecordService examRecordService;
    @Resource
    RedisUtil redisUtil;

    @Resource
    MqDtoService mqDtoService;

    @Resource
    MqUtil mqUtil;

    @Override
    public void persisted(Long recordId) {
        String lockKey = SystemConstant.REDIS_LOCK_EXAM_RECORD_PERSISTED_PREFIX + recordId;
        Boolean lock = redisUtil.lock(lockKey, SystemConstant.REDIS_CACHE_TIME_OUT);
        if (!lock) {
            log.info("persisted doing,request ignore recordId:" + recordId);
            return;
        }

        try {
            ExamRecordStatusEnum status = ExamRecordCacheUtil.getStatus(recordId);
            if (!ExamRecordStatusEnum.FINISHED.equals(status)) {
                log.info("status error ,status:" + status.name() + " recordId:" + recordId);
                return;
            }
            Double score = ExamRecordCacheUtil.getObjectiveScore(recordId);
            if (score == null) {
                log.info("score is null ,recordId:" + recordId);
                //算分未完成的 发送10秒延迟消息
                Map<String, Object> propMap = mqDtoService.buildMqDelayMsg("10s");
                MqDto mqDto = new MqDto(mqUtil.getMqGroupDomain().getTopic(), MqTagEnum.EXAM_RECORD_PERSISTED.name(),
                        Collections.singletonMap("recordId", recordId), MqTagEnum.EXAM_RECORD_PERSISTED, recordId.toString(), propMap, recordId.toString());
                mqDtoService.assembleSendAsyncDelayMsg(mqDto);
                return;
            }

            log.info("start persisted,recordId:" + recordId);
            examRecordService.saveDataByCache(recordId);
            // 清除缓存
//			ExamActivityRecordCacheUtil.removeActivityRecordCache(ExamRecordCacheUtil.getExamActivityId(recordId),
//					recordId);
            redisUtil.delete(RedisKeyHelper.examRecordCacheKey(recordId));
            redisUtil.delete(RedisKeyHelper.examAnswerKey(recordId));
            redisUtil.delete(RedisKeyHelper.audioLeftPlayCountKey(recordId));
            redisUtil.delete(RedisKeyHelper.livenessVerifyCacheKey(recordId));
            redisUtil.delete(RedisKeyHelper.faceVerifyCacheKey(recordId));
            log.info("end persisted,recordId:" + recordId);
        } finally {
            redisUtil.releaseLock(lockKey);
        }
    }

}
