package com.qmth.themis.business.service.impl;

import com.qmth.themis.business.bean.exam.QrMobileMonitorParamBean;
import com.qmth.themis.business.bean.exam.QrResponseBean;
import com.qmth.themis.business.bean.exam.QrUploadParamBean;
import com.qmth.themis.business.cache.MobileAuthCacheUtil;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.enums.MobileModeEnum;
import com.qmth.themis.business.enums.RoleEnum;
import com.qmth.themis.business.service.TEQrcodeService;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.business.util.WxappUtil;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.security.NoSuchAlgorithmException;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

@Service
public class TEQrcodeServiceImpl implements TEQrcodeService {

    @Resource
    RedisUtil redisUtil;

    @Resource
    WxappUtil wxappUtil;

    @Override
    public QrResponseBean mobileMonitor(QrMobileMonitorParamBean param) throws NoSuchAlgorithmException {
        String domain = wxappUtil.getWxappDomain().getUrl();
        String code = uuid();
        MobileModeEnum mode = MobileModeEnum.MOBILE_MONITOR;
        Set<String> roleCodes = new HashSet<>();
        roleCodes.add(RoleEnum.STUDENT.name());
        MobileAuthCacheUtil.setUserType(mode, code, roleCodes.toString());
        MobileAuthCacheUtil.setMode(mode, code);
        MobileAuthCacheUtil.setCode(mode, code);
        MobileAuthCacheUtil.setRecordId(mode, code, param.getRecordId());
        MobileAuthCacheUtil.setMonitorVideoSource(mode, code, param.getSource());
        Long expireTime = DateUtils.addSeconds(new Date(), SystemConstant.QR_EXPIRE_TIME.intValue()).getTime();
        redisUtil.expire(RedisKeyHelper.mobileAuthCacheKey(mode, code), SystemConstant.QR_EXPIRE_TIME.intValue());
        QrResponseBean ret = new QrResponseBean();
        ret.setExpireTime(expireTime);
        ret.setContent(domain + "/" + mode.name().toLowerCase() + "/" + code);
        return ret;
    }

    private String uuid() {
        return UUID.randomUUID().toString().replaceAll("-", "");
    }

    @Override
    public QrResponseBean upload(QrUploadParamBean param, MobileModeEnum mode) {
        String domain = wxappUtil.getWxappDomain().getUrl();
        String code = uuid();
        MobileAuthCacheUtil.setMode(mode, code);
        MobileAuthCacheUtil.setCode(mode, code);
        MobileAuthCacheUtil.setRecordId(mode, code, param.getRecordId());
        MobileAuthCacheUtil.setMainNumber(mode, code, param.getMainNumber());
        MobileAuthCacheUtil.setSubNumber(mode, code, param.getSubNumber());
        MobileAuthCacheUtil.setQuestionNumber(mode, code, param.getQuestionNumber());
        if (param.getSubIndex() != null) {
            MobileAuthCacheUtil.setSubIndex(mode, code, param.getSubIndex());
        }
        Long expireTime = DateUtils.addSeconds(new Date(), SystemConstant.QR_EXPIRE_TIME.intValue()).getTime();
        redisUtil.expire(RedisKeyHelper.mobileAuthCacheKey(mode, code), SystemConstant.QR_EXPIRE_TIME.intValue());
        QrResponseBean ret = new QrResponseBean();
        ret.setExpireTime(expireTime);
        ret.setContent(domain + "/" + mode.name().toLowerCase() + "/" + code);
        return ret;
    }

}
