package com.qmth.themis.business.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.themis.business.cache.ExamBreakCacheUtil;
import com.qmth.themis.business.cache.ExamRecordCacheUtil;
import com.qmth.themis.business.dao.TOeExamBreakHistoryMapper;
import com.qmth.themis.business.dto.MqDto;
import com.qmth.themis.business.entity.TOeExamBreakHistory;
import com.qmth.themis.business.enums.ExamBreakHistoryFieldEnum;
import com.qmth.themis.business.enums.MqTagEnum;
import com.qmth.themis.business.enums.VerifyExceptionEnum;
import com.qmth.themis.business.service.MqDtoService;
import com.qmth.themis.business.service.TOeExamBreakHistoryService;
import com.qmth.themis.business.util.MqUtil;
import com.qmth.themis.common.exception.BusinessException;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description: 考试断点记录 服务实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Service
public class TOeExamBreakHistoryServiceImpl extends ServiceImpl<TOeExamBreakHistoryMapper, TOeExamBreakHistory> implements TOeExamBreakHistoryService {

    @Resource
    MqUtil mqUtil;

    @Resource
    MqDtoService mqDtoService;

    /**
     * 保存验证信息
     *
     * @param recordId
     * @param entryAuthenticationId
     * @param entryAuthenticationResult
     */
    @Override
    public void saveAuthenticationResult(Long recordId, Long entryAuthenticationId, VerifyExceptionEnum entryAuthenticationResult) {
        Long examBreakId = ExamRecordCacheUtil.getLastBreakId(recordId);
        if (examBreakId == null) {
            throw new BusinessException("未找到断点ID");
        }
        if (ExamBreakCacheUtil.getExamRecordId(examBreakId) == null) {
            throw new BusinessException("未找到断点信息");
        }
        ExamBreakCacheUtil.setEntryAuthenticationId(examBreakId, entryAuthenticationId, false);
        ExamBreakCacheUtil.setEntryAuthenticationResult(examBreakId, entryAuthenticationResult, false);
        String[] columns = new String[]{ExamBreakHistoryFieldEnum.entry_authentication_id.name(),
                ExamBreakHistoryFieldEnum.entry_authentication_result.name()};
        Object[] values = new Object[]{entryAuthenticationId, entryAuthenticationResult.name()};
        this.dataUpdatesMq(examBreakId, columns, values);
    }

    @Override
    public void dataUpdateMq(Long breakId, String colName, Object colValue) {
        Map<String, Object> transMap = new HashMap<String, Object>();
        transMap.put("id", breakId);
        transMap.put("colName", colName);
        transMap.put("colValue", colValue);
        //mq发送消息start
        MqDto mqDto = new MqDto(mqUtil.getMqGroupDomain().getTopic(), MqTagEnum.EXAM_BREAK_HISTORY_UPDATE.name(), transMap,
                MqTagEnum.EXAM_BREAK_HISTORY_UPDATE, breakId.toString(), colName);
        mqDtoService.assembleSendOneWayMsg(mqDto);
    }

    @Override
    public void dataUpdatesMq(Long breakId, String[] colNames, Object[] colValues) {
        Map<String, Object> transMap = new HashMap<String, Object>();
        transMap.put("id", breakId);
        transMap.put("colNames", colNames);
        transMap.put("colValues", colValues);
        //mq发送消息start
        MqDto mqDto = new MqDto(mqUtil.getMqGroupDomain().getTopic(), MqTagEnum.EXAM_BREAK_HISTORY_UPDATE_COLUMNS.name(),
                transMap, MqTagEnum.EXAM_BREAK_HISTORY_UPDATE_COLUMNS, breakId.toString(), breakId.toString());
        mqDtoService.assembleSendOneWayMsg(mqDto);
    }
}
