package com.qmth.themis.business.util;

import com.qmth.themis.business.cache.ExamRecordCacheUtil;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.enums.ExamRecordFieldEnum;
import com.qmth.themis.business.enums.ExamRecordStatusEnum;
import com.qmth.themis.business.enums.WebsocketStatusEnum;
import com.qmth.themis.business.service.TOeExamRecordService;
import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: websocket util
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/27
 */
public class WebsocketUtil {
    private final static Logger log = LoggerFactory.getLogger(WebsocketUtil.class);

    public static Object getFieldInstance(Object obj, String fieldPath) {
        String fields[] = fieldPath.split("#");
        for (String field : fields) {
            obj = getField(obj, obj.getClass(), field);
            if (obj == null) {
                return null;
            }
        }

        return obj;
    }

    public static Object getField(Object obj, Class<?> clazz, String fieldName) {
        for (; clazz != Object.class; clazz = clazz.getSuperclass()) {
            try {
                Field field;
                field = clazz.getDeclaredField(fieldName);
                field.setAccessible(true);
                return field.get(obj);
            } catch (Exception e) {
            }
        }
        return null;
    }

    /**
     * 检查考试状态
     *
     * @param recordId
     */
    public static void checkExamStatus(Long recordId) {
        if (Objects.isNull(recordId)) {
            throw new BusinessException(ExceptionResultEnum.RECORD_ID_IS_NULL);
        }
        ExamRecordStatusEnum examRecordStatusEnum = ExamRecordCacheUtil.getStatus(recordId);
        if (Objects.isNull(examRecordStatusEnum)) {
            throw new BusinessException(ExceptionResultEnum.EXAM_STATUS_NOT_NULL);
        }
        if (!Objects.equals(ExamRecordStatusEnum.FIRST_PREPARE, examRecordStatusEnum) && !Objects.equals(ExamRecordStatusEnum.RESUME_PREPARE, examRecordStatusEnum)
                && !Objects.equals(ExamRecordStatusEnum.ANSWERING, examRecordStatusEnum)) {
            throw new BusinessException(ExceptionResultEnum.EXAM_STATUS_ERROR);
        }
    }

    /**
     * 更新websocket状态
     *
     * @param recordId
     * @param ip
     * @param websocketSessionId
     * @param websocketStatusEnum
     */
    public static void updateExamRecordWebsocketStatus(Long recordId, String ip, String websocketSessionId, WebsocketStatusEnum websocketStatusEnum) {
        ExamRecordCacheUtil.setClientWebsocketStatus(recordId, websocketStatusEnum, false);
        ExamRecordCacheUtil.setClientCurrentIp(recordId, ip, false);
        ExamRecordCacheUtil.setClientWebsocketId(recordId, websocketSessionId, false);
        Long clientLastSyncTime = System.currentTimeMillis();
        ExamRecordCacheUtil.setClientLastSyncTime(recordId, clientLastSyncTime, false);
        String[] columns = new String[]{ExamRecordFieldEnum.client_websocket_status.name(), ExamRecordFieldEnum.client_current_ip.name(), ExamRecordFieldEnum.client_websocket_id.name(), ExamRecordFieldEnum.client_last_sync_time.name()};
        Object[] values = new Object[]{websocketStatusEnum, ip, websocketSessionId, clientLastSyncTime};
        TOeExamRecordService tOeExamRecordService = SpringContextHolder.getBean(TOeExamRecordService.class);
        tOeExamRecordService.dataUpdatesMq(recordId, columns, values);
    }

    /**
     * 初始化websocket
     *
     * @param recordId
     * @param userId
     * @param deviceId
     * @param ip
     * @param updateTime
     * @return
     */
    public static Map<String, Object> initWebsocket(Long recordId, Long userId, String deviceId, String ip, Long updateTime) {
        Map<String, Object> tranMap = new HashMap<>();
        if (Objects.nonNull(recordId)) {
            tranMap.put("recordId", recordId);
        } else if (Objects.nonNull(userId)) {
            tranMap.put("userId", userId);
        }
        tranMap.put("deviceId", deviceId);
        tranMap.put("ip", ip);
        tranMap.put("updateTime", updateTime);
        return tranMap;
    }
}
